@@ -38,21 +38,21 @@
 #define INIT_SPIN(b, st, name, changed_notify)  init_spin(b, #name, G_STRUCT_OFFSET(st, name), changed_notify)
 #define INIT_ENTRY(b, st, name, changed_notify)  init_entry(b, #name, G_STRUCT_OFFSET(st, name), changed_notify)
 
-static GtkWidget* pref_dlg = NULL;
-static GtkWidget* notebook = NULL;
+static GtkWindow* pref_dlg = NULL;
+static GtkNotebook* notebook = NULL;
 /*
 static GtkWidget* icon_size_combo[3] = {0};
 static GtkWidget* bookmark_combo = NULL
 static GtkWidget* use_trash;
 */
 
-static GtkWidget* desktop_pref_dlg = NULL;
+static GtkWindow* desktop_pref_dlg = NULL;
 
-static void on_response(GtkDialog* dlg, int res, GtkWidget** pdlg)
+static void on_response(GtkDialog* dlg, int res, GtkWindow** pdlg)
 {
-    gtk_widget_destroy(GTK_WIDGET(dlg));
     *pdlg = NULL;
-    pcmanfm_save_config();
+    pcmanfm_save_config(TRUE);
+    gtk_widget_destroy(GTK_WIDGET(dlg));
 }
 
 static void on_icon_size_changed(GtkComboBox* combo, gpointer _off)
@@ -143,9 +143,9 @@ static void init_archiver_combo(GtkBuilder* builder)
     GtkListStore* model = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_POINTER);
     GtkComboBox* combo = (GtkComboBox*)gtk_builder_get_object(builder, "archiver");
     GtkTreeIter it;
-    GList* archivers = fm_archiver_get_all();
+    const GList* archivers = fm_archiver_get_all();
     FmArchiver* default_archiver = fm_archiver_get_default();
-    GList* l;
+    const GList* l;
 
     gtk_combo_box_set_model(combo, GTK_TREE_MODEL(model));
 
@@ -217,7 +217,7 @@ static void on_spin_changed(GtkSpinButton* btn, gpointer _off)
 {
     gsize off = GPOINTER_TO_SIZE(_off);
     guint* val = (guint*)G_STRUCT_MEMBER_P(fm_config, off);
-    gboolean new_val = gtk_spin_button_get_value(btn);
+    guint new_val = gtk_spin_button_get_value(btn);
     if(*val != new_val)
     {
         const char* name = g_object_get_data((GObject*)btn, "changed");
@@ -241,7 +241,7 @@ static void init_spin(GtkBuilder* b, const char* name, gsize off, const char* ch
 static void on_entry_changed(GtkEntry* entry, gpointer _off)
 {
     gsize off = GPOINTER_TO_SIZE(_off);
-    gchar** val = (guint*)G_STRUCT_MEMBER_P(fm_config, off);
+    gchar** val = (gchar**)G_STRUCT_MEMBER_P(fm_config, off);
     const char* new_val = gtk_entry_get_text(entry);
     if(g_strcmp0(*val, new_val))
     {
@@ -256,24 +256,40 @@ static void on_entry_changed(GtkEntry* entry, gpointer _off)
 
 static void init_entry(GtkBuilder* b, const char* name, gsize off, const char* changed_notify)
 {
-    GtkSpinButton* btn = GTK_SPIN_BUTTON(gtk_builder_get_object(b, name));
-    gchar** val = (guint*)G_STRUCT_MEMBER_P(fm_config, off);
+    GtkEntry* btn = GTK_ENTRY(gtk_builder_get_object(b, name));
+    gchar** val = (gchar**)G_STRUCT_MEMBER_P(fm_config, off);
     if(changed_notify)
         g_object_set_data_full(G_OBJECT(btn), "changed", g_strdup(changed_notify), g_free);
     if(*val)
-        gtk_entry_set_text(GTK_ENTRY(btn), *val);
+        gtk_entry_set_text(btn, *val);
     g_signal_connect(btn, "changed", G_CALLBACK(on_entry_changed), GSIZE_TO_POINTER(off));
 }
 
+static void on_tab_label_list_sel_changed(GtkTreeSelection* tree_sel, gpointer user_data)
+{
+    GtkTreePath* tp;
+    GtkTreeIter it;
+    GtkTreeModel* model;
+    gtk_tree_selection_get_selected(tree_sel, &model, &it);
+    tp = gtk_tree_model_get_path(model, &it);
+    gtk_notebook_set_current_page(notebook, gtk_tree_path_get_indices(tp)[0]);
+    gtk_tree_path_free(tp);
+}
+
 void fm_edit_preference( GtkWindow* parent, int page )
 {
     if(!pref_dlg)
     {
         GtkBuilder* builder = gtk_builder_new();
+        GtkTreeView* tab_label_list;
+        GtkTreeSelection* tree_sel;
+        GtkListStore* tab_label_model;
+        GtkTreeIter it;
+        int i, n;
 
         gtk_builder_add_from_file(builder, PACKAGE_UI_DIR "/pref.ui", NULL);
-        pref_dlg = gtk_builder_get_object(builder, "dlg");
-        notebook = gtk_builder_get_object(builder, "notebook");
+        pref_dlg = GTK_WINDOW(gtk_builder_get_object(builder, "dlg"));
+        notebook = GTK_NOTEBOOK(gtk_builder_get_object(builder, "notebook"));
 
         INIT_BOOL(builder, FmConfig, single_click, NULL);
         INIT_BOOL(builder, FmConfig, confirm_del, NULL);
@@ -305,14 +321,36 @@ void fm_edit_preference( GtkWindow* parent, int page )
         /* archiver integration */
         init_archiver_combo(builder);
 
+        /* initialize the left side list used for switching among tabs */
+        tab_label_list = GTK_TREE_VIEW(gtk_builder_get_object(builder, "tab_label_list"));
+        tab_label_model = gtk_list_store_new(1, G_TYPE_STRING);
+        n = gtk_notebook_get_n_pages(notebook);
+        for(i = 0; i < n; ++i)
+        {
+            /* this can be less efficient than iterating over a GList obtained by gtk_container_get_children().
+            * However, the order of pages does matter here. So we use get_nth_page. */
+            GtkWidget* page = gtk_notebook_get_nth_page(notebook, i);
+            const char* title = gtk_notebook_get_tab_label_text(notebook, page);
+            gtk_list_store_insert_with_values(tab_label_model, NULL, i, 0, title, -1);
+        }
+        gtk_tree_view_set_model(tab_label_list, GTK_TREE_MODEL(tab_label_model));
+        gtk_tree_model_get_iter_first(GTK_TREE_MODEL(tab_label_model), &it);
+        tree_sel = gtk_tree_view_get_selection(tab_label_list);
+        gtk_tree_selection_select_iter(tree_sel, &it);
+        g_object_unref(tab_label_model);
+        g_signal_connect(tree_sel, "changed", G_CALLBACK(on_tab_label_list_sel_changed), notebook);
+        gtk_notebook_set_show_tabs(notebook, FALSE);
+
         g_signal_connect(pref_dlg, "response", G_CALLBACK(on_response), &pref_dlg);
         g_object_unref(builder);
 
         pcmanfm_ref();
         g_signal_connect(pref_dlg, "destroy", G_CALLBACK(pcmanfm_unref), NULL);
+        if(parent)
+            gtk_window_set_transient_for(pref_dlg, parent);
     }
-    gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), page);
-    gtk_window_present(GTK_WINDOW(pref_dlg));
+    gtk_notebook_set_current_page(notebook, page);
+    gtk_window_present(pref_dlg);
 }
 
 static void on_wallpaper_set(GtkFileChooserButton* btn, gpointer user_data)
@@ -356,34 +394,35 @@ static void on_desktop_font_set(GtkFontButton* btn, gpointer user_data)
     }
 }
 
-void fm_desktop_preference()
+void fm_desktop_preference(GtkAction* act, GtkWindow* parent)
 {
     if(!desktop_pref_dlg)
     {
         GtkBuilder* builder = gtk_builder_new();
         GtkWidget* item, *img_preview;
         gtk_builder_add_from_file(builder, PACKAGE_UI_DIR "/desktop-pref.ui", NULL);
-        desktop_pref_dlg = gtk_builder_get_object(builder, "dlg");
+        desktop_pref_dlg = GTK_WINDOW(gtk_builder_get_object(builder, "dlg"));
 
-        item = gtk_builder_get_object(builder, "wallpaper");
+        item = (GtkWidget*)gtk_builder_get_object(builder, "wallpaper");
         g_signal_connect(item, "file-set", G_CALLBACK(on_wallpaper_set), NULL);
         img_preview = gtk_image_new();
         gtk_misc_set_alignment(GTK_MISC(img_preview), 0.5, 0.0);
         gtk_widget_set_size_request( img_preview, 128, 128 );
-        gtk_file_chooser_set_preview_widget( (GtkFileChooser*)item, img_preview );
+        gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(item), img_preview);
         g_signal_connect( item, "update-preview", G_CALLBACK(on_update_img_preview), img_preview );
         if(app_config->wallpaper)
             gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(item), app_config->wallpaper);
 
         INIT_COMBO(builder, FmAppConfig, wallpaper_mode, "wallpaper");
         INIT_COLOR(builder, FmAppConfig, desktop_bg, "wallpaper");
+        INIT_BOOL(builder, FmAppConfig, wallpaper_common, "wallpaper");
 
         INIT_COLOR(builder, FmAppConfig, desktop_fg, "desktop_text");
         INIT_COLOR(builder, FmAppConfig, desktop_shadow, "desktop_text");
 
         INIT_BOOL(builder, FmAppConfig, show_wm_menu, NULL);
 
-        item = gtk_builder_get_object(builder, "desktop_font");
+        item = (GtkWidget*)gtk_builder_get_object(builder, "desktop_font");
         if(app_config->desktop_font)
             gtk_font_button_set_font_name(GTK_FONT_BUTTON(item), app_config->desktop_font);
         g_signal_connect(item, "font-set", G_CALLBACK(on_desktop_font_set), NULL);
@@ -393,7 +432,8 @@ void fm_desktop_preference()
 
         pcmanfm_ref();
         g_signal_connect(desktop_pref_dlg, "destroy", G_CALLBACK(pcmanfm_unref), NULL);
+        if(parent)
+            gtk_window_set_transient_for(desktop_pref_dlg, parent);
     }
-    gtk_window_present(GTK_WINDOW(desktop_pref_dlg));
+    gtk_window_present(desktop_pref_dlg);
 }
-
